# -*- coding: utf-8 -*-
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from rest_framework import exceptions, viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from thx.api.views import ThuxViewMixin
from web.api.mixins import LightModelMixin
from . import serializers
from ... import models

STATUS_CODES = ((0, _('disable')), (1, _('enable')),)


# ORGANIZATION ADDRESS
class OrganizationAddressViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OrganizationAddress.objects.all()
    serializer_class = serializers.OrganizationAddressSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related(
            'organization', 'country',
        ).prefetch_related(
            'address_scopes'
        )
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'enable', 'disable', 'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'enable', 'disable', 'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'enable', 'disable', 'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'enable', 'disable', 'status_choices', 'light'
            ]:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in [
                'list', 'retrieve', 'status_choices', 'light'
            ]:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'status_choices', 'light'
            ]:
                return
        if user.is_patient and profile == 'patient':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'status_choices', 'light'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationAddress{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="address-type-choices")
    def address_type_choices(self, request, *args, **kwargs):
        return Response(settings.ADDRESS_ADDRESS_TYPE_CHOICES)


# EMAIL TYPOLOGY
class EmailTypologyViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.EmailTypology.objects.all()
    serializer_class = serializers.EmailTypologySerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'EmailTypology{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# PHONE TYPOLOGY
class PhoneTypologyViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.PhoneTypology.objects.all()
    serializer_class = serializers.PhoneTypologySerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'PhoneTypology{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# ORGANIZATION TYPE
class OrganizationTypeViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OrganizationType.objects.all()
    serializer_class = serializers.OrganizationTypeSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'category_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'category_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'category_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationType{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="category-choices")
    def category_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_ORGANIZATIONTYPE_CATEGORY_CHOICES)


# ORGANIZATION FORM
class OrganizationFormViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OrganizationForm.objects.all()
    serializer_class = serializers.OrganizationFormSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationForm{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# ORGANIZATION
class OrganizationViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.Organization.objects.all()
    serializer_class = serializers.OrganizationSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('organization_type', 'organization_form')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Organization{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# OWNER ORGANIZATION
class OwnerOrganizationViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OwnerOrganization.objects.all()
    serializer_class = serializers.OwnerOrganizationSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('organization_type', 'organization_form')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create',  'update', 'partial_update', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['retrieve']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['retrieve']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['retrieve']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['retrieve']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OwnerOrganization{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# ORGANIZATION EMAIL
class OrganizationEmailViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OrganizationEmail.objects.all()
    serializer_class = serializers.OrganizationEmailSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('organization', 'typology')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationEmail{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# ORGANIZATION PHONE
class OrganizationPhoneViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OrganizationPhone.objects.all()
    serializer_class = serializers.OrganizationPhoneSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('organization', 'typology')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationPhone{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# ORGANIZATION SOCIAL PAGE
class OrganizationSocialPageViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OrganizationSocialPage.objects.all()
    serializer_class = serializers.OrganizationSocialPageSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('organization')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'social_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'social_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'social_choices', 'light'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OrganizationSocialPage{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="social-choices")
    def social_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_ORGANIZATIONSOCIALPAGE_SOCIAL_DEF)


# CONTACT ROLE
class ContactRoleViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.ContactRole.objects.all()
    serializer_class = serializers.ContactRoleSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactRole{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# CONTACT TITLE
class ContactTitleViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.ContactTitle.objects.all()
    serializer_class = serializers.ContactTitleSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset()
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactTitle{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# CONTACT
class ContactViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.Contact.objects.all()
    serializer_class = serializers.ContactSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related(
            'user', 'organization', 'title', 'language'
        ).prefetch_related('roles')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'Contact{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# OWNER CONTACT
class OwnerContactViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.OwnerContact.objects.all()
    serializer_class = serializers.OwnerContactSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related(
            'user', 'organization', 'title', 'language'
        ).prefetch_related('roles')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'OwnerContact{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# CONTACT EMAIL
class ContactEmailViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.ContactEmail.objects.all()
    serializer_class = serializers.ContactEmailSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('contact', 'typology')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactEmail{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# CONTACT PHONE
class ContactPhoneViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.ContactPhone.objects.all()
    serializer_class = serializers.ContactPhoneSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('contact', 'typology')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactPhone{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)


# CONTACT SOCIAL ACCOUNT
class ContactSocialAccountViewSet(LightModelMixin, ThuxViewMixin, viewsets.ModelViewSet):
    queryset = models.ContactSocialAccount.objects.all()
    serializer_class = serializers.ContactSocialAccountSerializer

    def get_queryset(self):
        user = self.request.user
        profile = self.kwargs.get('profile')
        qs = super().get_queryset().select_related('contact')
        if user.is_superuser and profile == 'superuser':
            return qs
        if user.is_administrator and profile == 'administrator':
            return qs
        if user.is_manager and profile == 'manager':
            return qs
        if user.is_assistant and profile == 'assistant':
            return qs
        if user.is_pharma and profile == 'pharma':
            return qs.filter(status=1)
        if user.is_pharmacist and profile == 'pharmacist':
            return qs.filter(status=1)
        if user.is_doctor and profile == 'doctor':
            return qs.filter(status=1)
        if user.is_patient and profile == 'patient':
            return qs.filter(status=1)
        return qs.none()

    def check_permissions(self, request):
        super().check_permissions(request)
        user = self.request.user
        profile = self.kwargs.get('profile')
        if user.is_superuser and profile == 'superuser':
            return
        if user.is_administrator and profile == 'administrator':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_manager and profile == 'manager':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_assistant and profile == 'assistant':
            if self.action in [
                'list', 'retrieve', 'create', 'update', 'partial_update', 'destroy', 'enable', 'disable',
                'status_choices', 'light'
            ]:
                return
        if user.is_pharma and profile == 'pharma':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_pharmacist and profile == 'pharmacist':
            if self.action in ['list', 'retrieve']:
                return
        if user.is_doctor and profile == 'doctor':
            if self.action in ['list', 'retrieve', 'light']:
                return
        if user.is_patient and profile == 'patient':
            if self.action in ['list', 'retrieve', 'light']:
                return
        raise exceptions.PermissionDenied()

    def get_serializer_class(self):
        action = self.action.replace('-', ' ').replace('_', ' ').title().replace(' ', '')
        serializer = getattr(serializers, f'ContactSocialAccount{action}Serializer', None)
        if serializer:
            return serializer
        return super().get_serializer_class()

    @action(methods=['patch'], detail=True)
    def enable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['patch'], detail=True)
    def disable(self, request, **filters):
        return super().partial_update(request)

    @action(methods=['get'], detail=False, url_path="status-choices")
    def status_choices(self, request, *args, **kwargs):
        return Response(STATUS_CODES)

    @action(methods=['get'], detail=False, url_path="social-choices")
    def social_choices(self, request, *args, **kwargs):
        return Response(settings.ORGANIZATION_CONTACTSOCIALACCOUNT_SOCIAL_CHOICES)
